#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""extend base_image_id to 100 chars

Revision ID: 43df309dbf3f
Revises: 40df542e345
Create Date: 2015-06-30 20:39:56.988397

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '43df309dbf3f'
down_revision = '40df542e345'


def upgrade():
    op.alter_column('image', 'base_image_id',
                    type_=sa.String(length=100),
                    existing_type=sa.String(length=36),
                    existing_nullable=True,
                    )


def downgrade():
    op.alter_column('image', 'base_image_id',
                    type_=sa.String(length=36),
                    existing_type=sa.String(length=100),
                    existing_nullable=True,
                    )
