# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

ASSEM_URI_STR = '%s/camp/v1_1/assemblies/%s'

ASSEMS_URI_STR = '%s/camp/v1_1/assemblies'

ATTRIBUTE_DEF_URI_STR = '%s/camp/v1_1/attribute_definitions/%s'

DEPLOY_PARAMS_URI = '%s/camp/v1_1/parameter_definitions/deploy_params'

EXTN_URI_STR = '%s/v1/extensions/%s'

EXTNS_URI_STR = '%s/camp/v1_1/extensions'

FORMAT_URI_STR = '%s/camp/v1_1/formats/%s'

FORMATS_URI_STR = '%s/camp/v1_1/formats'

JSON_FORMAT_URI_STR = '%s/camp/v1_1/formats/json_format'
JSON_FORMAT_NAME_STR = 'JSON'

PARAM_DEF_URI_STR = '%s/camp/v1_1/parameter_definition/%s'
PARAM_DEFS_URI_STR = '%s/camp/v1_1/parameter_definitions/%s'

PLAN_URI_STR = '%s/camp/v1_1/plans/%s'

PLANS_URI_STR = '%s/camp/v1_1/plans'

PLATFORM_URI_STR = '%s/camp/v1_1/platform'

SERV_URI_STR = '%s/v1/services/%s'

SERVS_URI_STR = '%s/camp/v1_1/services'

TYPE_DEF_URI_STR = '%s/camp/v1_1/type_definitions/%s'
TYPE_DEFS_URI_STR = '%s/camp/v1_1/type_definitions'
